'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _easings = require('./easings');

var easings = _interopRequireWildcard(_easings);

var _utils = require('./utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js Camera Class
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * ======================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Class designed to store camera information & used to update it.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


/**
 * Defaults.
 */
var ANIMATE_DEFAULTS = {
  easing: 'quadraticInOut',
  duration: 150
};

var DEFAULT_ZOOMING_RATIO = 1.5;

// TODO: animate options = number polymorphism?
// TODO: pan, zoom, unzoom, reset, rotate, zoomTo
// TODO: add width / height to camera and add #.resize
// TODO: bind camera to renderer rather than sigma
// TODO: add #.graphToDisplay, #.displayToGraph, batch methods later

/**
 * Camera class
 *
 * @constructor
 */

var Camera = function (_EventEmitter) {
  _inherits(Camera, _EventEmitter);

  function Camera(dimensions) {
    _classCallCheck(this, Camera);

    dimensions = dimensions || {};

    // Properties
    var _this = _possibleConstructorReturn(this, (Camera.__proto__ || Object.getPrototypeOf(Camera)).call(this));

    _this.x = 0;
    _this.y = 0;
    _this.angle = 0;
    _this.ratio = 1;
    _this.width = dimensions.width || 0;
    _this.height = dimensions.height || 0;

    // State
    _this.nextFrame = null;
    _this.enabled = true;
    return _this;
  }

  /**
   * Method used to enable the camera.
   *
   * @return {Camera}
   */


  _createClass(Camera, [{
    key: 'enable',
    value: function enable() {
      this.enabled = true;
      return this;
    }

    /**
     * Method used to disable the camera.
     *
     * @return {Camera}
     */

  }, {
    key: 'disable',
    value: function disable() {
      this.enabled = false;
      return this;
    }

    /**
     * Method used to retrieve the camera's current state.
     *
     * @return {object}
     */

  }, {
    key: 'getState',
    value: function getState() {
      return {
        x: this.x,
        y: this.y,
        angle: this.angle,
        ratio: this.ratio
      };
    }

    /**
     * Method used to retrieve the camera's dimensions.
     *
     * @return {object}
     */

  }, {
    key: 'getDimensions',
    value: function getDimensions() {
      return {
        width: this.width,
        height: this.height
      };
    }

    /**
     * Method used to check whether the camera is currently being animated.
     *
     * @return {boolean}
     */

  }, {
    key: 'isAnimated',
    value: function isAnimated() {
      return !!this.nextFrame;
    }

    /**
     * Method returning the coordinates of a point from the display frame to the
     * graph one but ignores the offset and dimensions of the container.
     *
     * @param  {number} x - The X coordinate.
     * @param  {number} y - The Y coordinate.
     * @return {object}     The point coordinates in the frame of the graph.
     */

  }, {
    key: 'abstractDisplayToGraph',
    value: function abstractDisplayToGraph(x, y) {
      var cos = Math.cos(this.angle),
          sin = Math.sin(this.angle);

      return {
        x: (x * cos - y * sin) * this.ratio,
        y: (y * cos + x * sin) * this.ratio
      };
    }

    /**
     * Method returning the coordinates of a point from the display frame to the
     * graph one.
     *
     * @param  {number} x - The X coordinate.
     * @param  {number} y - The Y coordinate.
     * @return {object}   - The point coordinates in the frame of the graph.
     */

  }, {
    key: 'displayToGraph',
    value: function displayToGraph(x, y) {
      var cos = Math.cos(this.angle),
          sin = Math.sin(this.angle);

      var xOffset = this.width / 2 - (this.x * cos + this.y * sin) / this.ratio,
          yOffset = this.height / 2 - (this.y * cos - this.x * sin) / this.ratio;

      var X = x - xOffset,
          Y = y - yOffset;

      return {
        x: (X * cos - Y * sin) * this.ratio,
        y: (Y * cos + X * sin) * this.ratio
      };
    }

    /**
     * Method returning the coordinates of a point from the graph frame to the
     * display one.
     *
     * @param  {number} x - The X coordinate.
     * @param  {number} y - The Y coordinate.
     * @return {object}   - The point coordinates in the frame of the display.
     */

  }, {
    key: 'graphToDisplay',
    value: function graphToDisplay(x, y) {
      var relCos = Math.cos(this.angle) / this.ratio,
          relSin = Math.sin(this.angle) / this.ratio,
          xOffset = this.width / 2 - this.x * relCos - this.y * relSin,
          yOffset = this.height / 2 - this.y * relCos + this.x * relSin;

      return {
        x: x * relCos + y * relSin + xOffset,
        y: y * relCos - x * relSin + yOffset
      };
    }

    /**
     * Method returning the abstract rectangle containing the graph according
     * to the camera's state.
     *
     * @return {object} - The view's rectangle.
     */

  }, {
    key: 'viewRectangle',
    value: function viewRectangle() {
      var widthVect = this.abstractDisplayToGraph(this.width, 0),
          heightVect = this.abstractDisplayToGraph(0, this.height),
          centerVect = this.abstractDisplayToGraph(this.width / 2, this.height / 2),
          marginX = this.abstractDisplayToGraph(this.width / 4, 0).x,
          marginY = this.abstractDisplayToGraph(0, this.height / 4, 0).y;

      return {
        x1: this.x - centerVect.x - marginX,
        y1: this.y - centerVect.y - marginY,
        x2: this.x - centerVect.x + marginX + widthVect.x,
        y2: this.y - centerVect.y - marginY + widthVect.y,
        height: Math.sqrt(Math.pow(heightVect.x, 2) + Math.pow(heightVect.y + 2 * marginY, 2))
      };
    }

    /**
     * Method used to set the camera's state.
     *
     * @param  {object} state - New state.
     * @return {Camera}
     */

  }, {
    key: 'setState',
    value: function setState(state) {

      if (!this.enabled) return this;

      // TODO: validations
      // TODO: update by function

      if ('x' in state) this.x = state.x;

      if ('y' in state) this.y = state.y;

      if ('angle' in state) this.angle = state.angle;

      if ('ratio' in state) this.ratio = state.ratio;

      // Emitting
      // TODO: don't emit if nothing changed?
      this.emit('updated', this.getState());

      return this;
    }

    /**
     * Method used to resize the camera's dimensions.
     *
     * @param  {object} dimensions - New dimensions.
     * @return {Camera}
     */

  }, {
    key: 'resize',
    value: function resize(dimensions) {

      if (!this.enabled) return this;

      if ('width' in dimensions) this.width = dimensions.width;

      if ('height' in dimensions) this.height = dimensions.height;

      this.emit('resized', this.getDimensions());

      return this;
    }

    /**
     * Method used to animate the camera.
     *
     * @param  {object}   state      - State to reach eventually.
     * @param  {object}   options    - Options:
     * @param  {number}     duration - Duration of the animation.
     * @param  {function} callback   - Callback
     * @return {function}            - Return a function to cancel the animation.
     */

  }, {
    key: 'animate',
    value: function animate(state, options, callback) {
      var _this2 = this;

      if (!this.enabled) return this;

      // TODO: validation

      options = (0, _utils.assign)({}, ANIMATE_DEFAULTS, options);

      var easing = typeof options.easing === 'function' ? options.easing : easings[options.easing];

      // Canceling previous animation if needed
      if (this.nextFrame) cancelAnimationFrame(this.nextFrame);

      // State
      var start = Date.now(),
          initialState = this.getState();

      // Function performing the animation
      var fn = function fn() {
        var t = (Date.now() - start) / options.duration;

        // The animation is over:
        if (t >= 1) {
          _this2.nextFrame = null;
          _this2.setState(state);

          if (typeof callback === 'function') callback();

          return;
        }

        var coefficient = easing(t);

        var newState = {};

        if ('x' in state) newState.x = initialState.x + (state.x - initialState.x) * coefficient;
        if ('y' in state) newState.y = initialState.y + (state.y - initialState.y) * coefficient;
        if ('angle' in state) newState.angle = initialState.angle + (state.angle - initialState.angle) * coefficient;
        if ('ratio' in state) newState.ratio = initialState.ratio + (state.ratio - initialState.ratio) * coefficient;

        _this2.setState(newState);

        _this2.nextFrame = requestAnimationFrame(fn);
      };

      if (this.nextFrame) {
        cancelAnimationFrame(this.nextFrame);
        this.nextFrame = requestAnimationFrame(fn);
      } else {
        fn();
      }
    }

    /**
     * Method used to zoom the camera.
     *
     * @param  {number|object} factorOrOptions - Factor or options.
     * @return {function}
     */

  }, {
    key: 'animatedZoom',
    value: function animatedZoom(factorOrOptions) {

      if (!factorOrOptions) {
        return this.animate({ ratio: this.ratio / DEFAULT_ZOOMING_RATIO });
      } else {
        if (typeof factorOrOptions === 'number') return this.animate({ ratio: this.ratio / factorOrOptions });else return this.animate({ ratio: this.ratio / (factorOrOptions.factor || DEFAULT_ZOOMING_RATIO) }, factorOrOptions);
      }
    }

    /**
     * Method used to unzoom the camera.
     *
     * @param  {number|object} factorOrOptions - Factor or options.
     * @return {function}
     */

  }, {
    key: 'animatedUnzoom',
    value: function animatedUnzoom(factorOrOptions) {

      if (!factorOrOptions) {
        return this.animate({ ratio: this.ratio * DEFAULT_ZOOMING_RATIO });
      } else {
        if (typeof factorOrOptions === 'number') return this.animate({ ratio: this.ratio * factorOrOptions });else return this.animate({ ratio: this.ratio * (factorOrOptions.factor || DEFAULT_ZOOMING_RATIO) }, factorOrOptions);
      }
    }

    /**
     * Method used to reset the camera.
     *
     * @param  {object} options - Options.
     * @return {function}
     */

  }, {
    key: 'animatedReset',
    value: function animatedReset(options) {
      return this.animate({
        x: 0,
        y: 0,
        ratio: 1,
        angle: 0
      }, options);
    }
  }]);

  return Camera;
}(_events.EventEmitter);

exports.default = Camera;