'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _captor = require('../captor');

var _captor2 = _interopRequireDefault(_captor);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js Mouse Captor
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * ======================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma's captor dealing with the user's mouse.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


/**
 * Constants.
 */
var DRAG_TIMEOUT = 200,
    MOUSE_INERTIA_DURATION = 200,
    MOUSE_INERTIA_RATIO = 3,
    MOUSE_ZOOM_DURATION = 200,
    ZOOMING_RATIO = 1.7,
    DOUBLE_CLICK_TIMEOUT = 300,
    DOUBLE_CLICK_ZOOMING_RATIO = 2.2,
    DOUBLE_CLICK_ZOOMING_DURATION = 200;

/**
 * Mouse captor class.
 *
 * @constructor
 */

var MouseCaptor = function (_Captor) {
  _inherits(MouseCaptor, _Captor);

  function MouseCaptor(container, camera) {
    _classCallCheck(this, MouseCaptor);

    // Properties
    var _this = _possibleConstructorReturn(this, (MouseCaptor.__proto__ || Object.getPrototypeOf(MouseCaptor)).call(this, container, camera));

    _this.container = container;
    _this.camera = camera;

    // State
    _this.enabled = true;
    _this.hasDragged = false;
    _this.downStartTime = null;
    _this.startMouseX = null;
    _this.startMouseY = null;
    _this.isMouseDown = false;
    _this.isMoving = false;
    _this.movingTimeout = null;
    _this.startCameraState = null;
    _this.lastCameraState = null;
    _this.clicks = 0;
    _this.doubleClickTimeout = null;
    _this.wheelLock = false;

    // Binding methods
    _this.handleClick = _this.handleClick.bind(_this);
    _this.handleDown = _this.handleDown.bind(_this);
    _this.handleUp = _this.handleUp.bind(_this);
    _this.handleMove = _this.handleMove.bind(_this);
    _this.handleWheel = _this.handleWheel.bind(_this);
    _this.handleOut = _this.handleOut.bind(_this);

    // Binding events
    container.addEventListener('click', _this.handleClick, false);
    container.addEventListener('mousedown', _this.handleDown, false);
    container.addEventListener('mousemove', _this.handleMove, false);
    container.addEventListener('DOMMouseScroll', _this.handleWheel, false);
    container.addEventListener('mousewheel', _this.handleWheel, false);
    container.addEventListener('mouseout', _this.handleOut, false);

    document.addEventListener('mouseup', _this.handleUp, false);
    return _this;
  }

  _createClass(MouseCaptor, [{
    key: 'handleClick',
    value: function handleClick(e) {
      var _this2 = this;

      if (!this.enabled) return;

      this.clicks++;

      if (this.clicks === 2) {
        this.clicks = 0;

        clearTimeout(this.doubleClickTimeout);
        this.doubleClickTimeout = null;

        return this.handleDoubleClick(e);
      }

      setTimeout(function () {
        _this2.clicks = 0;
        _this2.doubleClickTimeout = null;
      }, DOUBLE_CLICK_TIMEOUT);

      this.emit('click', (0, _utils.getMouseCoords)(e));
    }
  }, {
    key: 'handleDoubleClick',
    value: function handleDoubleClick(e) {
      if (!this.enabled) return;

      var ratio = 1 / DOUBLE_CLICK_ZOOMING_RATIO;

      var center = (0, _utils.getCenter)(e);

      var cameraState = this.camera.getState();

      var position = this.camera.abstractDisplayToGraph((0, _utils.getX)(e) - center.x, (0, _utils.getY)(e) - center.y);

      this.camera.animate({
        x: position.x * (1 - ratio) + cameraState.x,
        y: position.y * (1 - ratio) + cameraState.y,
        ratio: ratio * cameraState.ratio
      }, {
        easing: 'quadraticInOut',
        duration: DOUBLE_CLICK_ZOOMING_DURATION
      });

      if (e.preventDefault) e.preventDefault();else e.returnValue = false;

      e.stopPropagation();

      return false;
    }
  }, {
    key: 'handleDown',
    value: function handleDown(e) {
      if (!this.enabled) return;

      this.startCameraState = this.camera.getState();
      this.lastCameraState = this.startCameraState;

      this.startMouseX = (0, _utils.getX)(e);
      this.startMouseY = (0, _utils.getY)(e);

      this.hasDragged = false;

      this.downStartTime = Date.now();

      // TODO: dispatch events
      switch (e.which) {
        default:

          // Left button pressed
          this.isMouseDown = true;
          this.emit('mousedown', (0, _utils.getMouseCoords)(e));
      }
    }
  }, {
    key: 'handleUp',
    value: function handleUp(e) {
      if (!this.enabled || !this.isMouseDown) return;

      this.isMouseDown = false;

      if (this.movingTimeout) {
        this.movingTimeout = null;
        clearTimeout(this.movingTimeout);
      }

      var x = (0, _utils.getX)(e),
          y = (0, _utils.getY)(e);

      var cameraState = this.camera.getState();

      if (this.isMoving) {
        this.camera.animate({
          x: cameraState.x + MOUSE_INERTIA_RATIO * (cameraState.x - this.lastCameraState.x),
          y: cameraState.y + MOUSE_INERTIA_RATIO * (cameraState.y - this.lastCameraState.y)
        }, {
          duration: MOUSE_INERTIA_DURATION,
          easing: 'quadraticOut'
        });
      } else if (this.startMouseX !== x || this.startMouseY !== y) {
        this.camera.setState({
          x: cameraState.x,
          y: cameraState.y
        });
      }

      this.isMoving = false;
      this.hasDragged = false;
      this.emit('mouseup', (0, _utils.getMouseCoords)(e));
    }
  }, {
    key: 'handleMove',
    value: function handleMove(e) {
      var _this3 = this;

      if (!this.enabled) return;

      this.emit('mousemove', (0, _utils.getMouseCoords)(e));

      if (this.isMouseDown) {

        // TODO: dispatch events
        this.isMoving = true;
        this.hasDragged = true;

        if (this.movingTimeout) clearTimeout(this.movingTimeout);

        this.movingTimeout = setTimeout(function () {
          _this3.movingTimeout = null;
          _this3.isMoving = false;
        }, DRAG_TIMEOUT);

        var position = this.camera.abstractDisplayToGraph((0, _utils.getX)(e) - this.startMouseX, (0, _utils.getY)(e) - this.startMouseY);

        var x = this.startCameraState.x - position.x,
            y = this.startCameraState.y - position.y;

        var cameraState = this.camera.getState();

        if (cameraState.x !== x || cameraState.y !== y) {

          this.lastCameraState = cameraState;

          this.camera.setState({
            x: x,
            y: y
          });
        }
      }

      if (e.preventDefault) e.preventDefault();else e.returnValue = false;

      e.stopPropagation();

      return false;
    }
  }, {
    key: 'handleWheel',
    value: function handleWheel(e) {
      var _this4 = this;

      if (e.preventDefault) e.preventDefault();else e.returnValue = false;

      e.stopPropagation();

      if (!this.enabled) return false;

      var delta = (0, _utils.getWheelDelta)(e);

      if (!delta) return false;

      if (this.wheelLock) return false;

      this.wheelLock = true;

      // TODO: handle max zoom
      var ratio = delta > 0 ? 1 / ZOOMING_RATIO : ZOOMING_RATIO;

      var cameraState = this.camera.getState();

      var newRatio = ratio * cameraState.ratio;

      var center = (0, _utils.getCenter)(e);

      var position = this.camera.abstractDisplayToGraph((0, _utils.getX)(e) - center.x, (0, _utils.getY)(e) - center.y);

      this.camera.animate({
        x: position.x * (1 - ratio) + cameraState.x,
        y: position.y * (1 - ratio) + cameraState.y,
        ratio: newRatio
      }, {
        easing: 'linear',
        duration: MOUSE_ZOOM_DURATION
      }, function () {
        return _this4.wheelLock = false;
      });

      return false;
    }
  }, {
    key: 'handleOut',
    value: function handleOut() {
      // TODO: dispatch event
    }
  }]);

  return MouseCaptor;
}(_captor2.default);

exports.default = MouseCaptor;