'use strict';

var _camera = require('../camera');

var _camera2 = _interopRequireDefault(_camera);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_CELL = {
  width: 200,
  height: 150
}; /**
    * Sigma.js Labels Heuristics
    * ===========================
    *
    * Heuristics related to label display.
    */


var DEFAULT_UNZOOMED_CELL = {
  width: 400,
  height: 300
};

var DEFAULT_CELL_POOL = 1;

// TODO: possible to use threshold (should be applied by quadtree for perf)
exports.getLabelsToDisplay = function (camera, lastCameraState, cache, nodeOrder, visibleNodes, lastVisibleNodes, displayedLabels, params) {

  // TODO: probable memory leak on displayed label when panning when zoomed

  var cameraState = camera.getState(),
      dimensions = camera.getDimensions();

  var centeredCamera = new _camera2.default(dimensions);
  // centeredCamera.setState({ratio: cameraState.ratio});

  // console.log(offset)
  // const oldCamera = new Camera(dimensions);
  // oldCamera.setState(lastCameraState);
  centeredCamera.setState({ ratio: cameraState.ratio });

  var zooming = cameraState.ratio < lastCameraState.ratio,
      panning = cameraState.x !== lastCameraState.x || cameraState.y !== lastCameraState.x,
      unzooming = cameraState.ratio > lastCameraState.ratio,
      unzoomedPanning = !zooming && !unzooming && cameraState.ratio >= 1;

  var baseCell = cameraState.ratio >= 1.3 ? DEFAULT_UNZOOMED_CELL : DEFAULT_CELL;

  // If we are panning while unzoomed, we shouldn't change label selection
  if (unzoomedPanning && displayedLabels.size !== 0) return Array.from(displayedLabels);

  // Adapting the cellWidth
  var cellWidthRemainder = dimensions.width % baseCell.width,
      cellWidth = baseCell.width + cellWidthRemainder / Math.floor(dimensions.width / baseCell.width);

  var cellHeightRemainder = dimensions.height % baseCell.height,
      cellHeight = baseCell.height + cellWidthRemainder / Math.floor(dimensions.height / baseCell.height);

  // Building the grid
  var grid = {};

  var worthyBuckets = new Set();
  var worthyLabels = zooming || panning && !unzooming ? Array.from(displayedLabels) : [];

  for (var i = 0, l = visibleNodes.length; i < l; i++) {
    var node = visibleNodes[i],
        data = cache[node];

    if (panning && !zooming && !unzooming) {
      if (!displayedLabels.has(node) && lastVisibleNodes.has(node)) continue;
    }

    var pos = camera.graphToDisplay(data.x, data.y);

    var centeredPos = centeredCamera.graphToDisplay(data.x, data.y);

    // Filtering out-of-view nodes (quadtree quirk)
    // TODO: this should probably be done by quadtree beforehand?
    // if (
    //   (pos.x < 0 || pos.x > dimensions.width) ||
    //   (pos.y < 0 || pos.y > dimensions.height)
    // )
    //   continue;

    // TODO: filter negative buckets and further than required
    var x = Math.floor(pos.x / cellWidth) % cellWidth,
        y = Math.floor(pos.y / cellHeight) % cellHeight;

    var key = x + ';' + y;

    var bucket = grid[key];

    if (unzooming && worthyBuckets.has(key)) {
      if (!displayedLabels.has(node)) continue;

      var championData = cache[bucket[0]];

      if (data.size > championData.size) {
        bucket[0] = node;
      }

      if (data.size === championData.size) {
        var challengerOrder = nodeOrder[node],
            championOrder = nodeOrder[bucket[0]];

        if (challengerOrder < championOrder) bucket[0] = node;
      }

      continue;
    }

    if (typeof bucket === 'undefined') {
      bucket = [];
      grid[key] = bucket;
    }

    if (displayedLabels.has(node)) worthyBuckets.add(key);

    bucket.push(node);
  }

  // Selecting worthy labels
  for (var _key in grid) {
    var _bucket = grid[_key];

    if ((zooming || panning && !unzooming) && worthyBuckets.has(_key)) continue;

    // TODO: if we keep the only 1 label per cell, we don't need sorting
    // it's just a matter of keeping the max above
    _bucket.sort(function (a, b) {
      // const aDisplayed = displayedLabels.has(a),
      //       bDisplayed = displayedLabels.has(b);

      var aData = cache[a],
          bData = cache[b];

      // if (aDisplayed < bDisplayed)
      //   return -1;

      // if (aDisplayed > bDisplayed)
      //   return 1;

      if (aData.size < bData.size) return 1;

      if (aData.size > bData.size) return -1;

      var aOrder = nodeOrder[a],
          bOrder = nodeOrder[b];

      if (aOrder < bOrder) return 1;

      if (aOrder > bOrder) return -1;

      return 0;
    });

    var _i = 0,
        _l = _bucket.length;

    while (_i < DEFAULT_CELL_POOL && _i < _l) {
      var _node = _bucket[_i];

      worthyLabels.push(_node);
      _i++;
    }
  }

  return worthyLabels;
};