'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Sigma.js Core Class
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * ====================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Core class holding state for the bound graph and using a combination of
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * a renderer & camera to display the bound graph on screen.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _isGraph = require('graphology-utils/is-graph');

var _isGraph2 = _interopRequireDefault(_isGraph);

var _renderer = require('./renderer');

var _renderer2 = _interopRequireDefault(_renderer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// TODO: sigma should only hold the graph and compose state to give to renderer,
// does it need to orchestrate?

// TODO: should be able to take n renderers
// TODO: what to do with refresh methods? We should probably drop them

// TODO: create a thumbnail renderer

/**
 * Sigma class
 *
 * @constructor
 * @param {Graph}    graph    - A graphology Graph instance.
 * @param {Renderer} renderer - A Renderer instance.
 */
var Sigma = function () {
  function Sigma(graph, renderer) {
    _classCallCheck(this, Sigma);

    // Checking the arguments
    if (!(0, _isGraph2.default)(graph)) throw new Error('Sigma.constructor: given graph is not an instance of a graphology implementation.');

    if (!(renderer instanceof _renderer2.default)) throw new Error('Sigma.constructor: given renderer is not an instance of a sigma Renderer.');

    // Properties
    this.graph = graph;
    this.renderer = renderer;
    this.renderer.bind(this);

    // Userland state
    this.state = {};
    this.nodeStates = null;
    this.edgeStates = null;

    // First time render
    this.renderer.render();
  }

  /**---------------------------------------------------------------------------
   * Internals
   **---------------------------------------------------------------------------
   */

  /**---------------------------------------------------------------------------
   * Getters
   **---------------------------------------------------------------------------
   */

  /**
   * Method returning the graph bound to the instance.
   *
   * @return {Graph} - The bound graph.
   */


  _createClass(Sigma, [{
    key: 'getGraph',
    value: function getGraph() {
      return this.graph;
    }

    /**
     * Method returning the composed data of the target node.
     *
     * @return {string} key - The node's key.
     * @return {object}     - The node's attributes.
     */

  }, {
    key: 'getNodeData',
    value: function getNodeData(key) {

      // TODO: this will change to compose state later
      return this.graph.getNodeAttributes(key);
    }

    /**
     * Method returning the composed data of the target edge.
     *
     * @return {string} key - The edge's key.
     * @return {object}     - The edge's attributes.
     */

  }, {
    key: 'getEdgeData',
    value: function getEdgeData(key) {

      // TODO: this will change to compose state later
      return this.graph.getEdgeAttributes(key);
    }

    /**
     * Method returning the extent of the bound graph.
     *
     * @return {object} - The graph's extent.
     */

  }, {
    key: 'getGraphExtent',
    value: function getGraphExtent() {
      var graph = this.graph;

      var nodes = graph.nodes(),
          edges = graph.edges();

      var maxX = -Infinity,
          maxY = -Infinity,
          minX = Infinity,
          minY = Infinity,
          maxNodeSize = -Infinity,
          maxEdgeSize = -Infinity;

      for (var i = 0, l = nodes.length; i < l; i++) {
        var node = nodes[i];

        var data = this.getNodeData(node);

        var size = data.size || 1;

        if (data.x > maxX) maxX = data.x;
        if (data.y > maxY) maxY = data.y;

        if (data.x < minX) minX = data.x;
        if (data.y < minY) minY = data.y;

        if (size > maxNodeSize) maxNodeSize = size;
      }

      for (var _i = 0, _l = edges.length; _i < _l; _i++) {
        var edge = edges[_i];

        var _data = this.getEdgeData(edge);

        var _size = _data.size || 1;

        if (_size > maxEdgeSize) maxEdgeSize = _size;
      }

      return {
        maxX: maxX,
        maxY: maxY,
        minX: minX,
        minY: minY,
        maxNodeSize: maxNodeSize,
        maxEdgeSize: maxEdgeSize
      };
    }
  }]);

  return Sigma;
}();

exports.default = Sigma;